@@ -119,11 +119,21 @@ fn emit_codespan(
         )
         .unwrap();
     } else {
-        codespan_reporting::term::emit(writer, &config, files, diagnostic)
+        codespan_reporting::term::emit(writer, config, files, diagnostic)
             .expect("couldn't emit error to codespan");
     }
 }
 
+fn emit_codespan_locked(
+    files: &codespan::Files<&str>,
+    diagnostic: &CodespanDiagnostic<codespan::FileId>,
+) {
+    let stdout = termcolor::StandardStream::stdout(get_color());
+    let mut stdout = stdout.lock();
+
+    emit_codespan(&mut stdout, files, diagnostic);
+}
+
 fn read<R: Read>(checker: &Checker<toml::value::Value>, filename: &Path, mut reader: R) {
     let mut buffer = Vec::new();
     if let Err(error) = reader.read_to_end(&mut buffer) {
@@ -145,21 +155,16 @@ fn read<R: Read>(checker: &Checker<toml::value::Value>, filename: &Path, mut rea
     let mut files = codespan::Files::new();
     let source_id = files.add(filename.as_os_str(), &*contents);
 
-    let stdout = termcolor::StandardStream::stdout(get_color());
-    let mut stdout = stdout.lock();
-
     let ast = match full_moon::parse(&contents) {
         Ok(ast) => ast,
         Err(error) => {
             PARSE_ERRORS.fetch_add(1, Ordering::SeqCst);
 
-            if let full_moon::Error::AstError(full_moon::ast::AstError::UnexpectedToken {
-                token,
-                additional,
-            }) = error
-            {
-                emit_codespan(
-                    &mut stdout,
+            match error {
+                full_moon::Error::AstError(full_moon::ast::AstError::UnexpectedToken {
+                    token,
+                    additional,
+                }) => emit_codespan_locked(
                     &files,
                     &CodespanDiagnostic {
                         severity: CodespanSeverity::Error,
@@ -175,9 +180,42 @@ fn read<R: Read>(checker: &Checker<toml::value::Value>, filename: &Path, mut rea
                         .with_message(additional.unwrap_or_default())],
                         notes: Vec::new(),
                     },
-                );
-            } else {
-                error!("Error parsing {}: {}", filename.display(), error);
+                ),
+                full_moon::Error::TokenizerError(error) => emit_codespan_locked(
+                    &files,
+                    &CodespanDiagnostic {
+                        severity: CodespanSeverity::Error,
+                        code: Some("parse_error".to_owned()),
+                        message: match error.error() {
+                            full_moon::tokenizer::TokenizerErrorType::UnclosedComment => {
+                                "unclosed comment".to_string()
+                            }
+                            full_moon::tokenizer::TokenizerErrorType::UnclosedString => {
+                                "unclosed string".to_string()
+                            }
+                            full_moon::tokenizer::TokenizerErrorType::UnexpectedShebang => {
+                                "unexpected shebang".to_string()
+                            }
+                            full_moon::tokenizer::TokenizerErrorType::UnexpectedToken(
+                                character,
+                            ) => {
+                                format!("unexpected character {}", character)
+                            }
+                            full_moon::tokenizer::TokenizerErrorType::InvalidSymbol(symbol) => {
+                                format!("invalid symbol {}", symbol)
+                            }
+                        },
+                        labels: vec![CodespanLabel::primary(
+                            source_id,
+                            codespan::Span::new(
+                                error.position().bytes() as u32,
+                                error.position().bytes() as u32,
+                            ),
+                        )],
+                        notes: Vec::new(),
+                    },
+                ),
+                _ => error!("Error parsing {}: {}", filename.display(), error),
             }
 
             return;
@@ -199,6 +237,9 @@ fn read<R: Read>(checker: &Checker<toml::value::Value>, filename: &Path, mut rea
     LINT_ERRORS.fetch_add(errors, Ordering::SeqCst);
     LINT_WARNINGS.fetch_add(warnings, Ordering::SeqCst);
 
+    let stdout = termcolor::StandardStream::stdout(get_color());
+    let mut stdout = stdout.lock();
+
     for diagnostic in diagnostics {
         if opts.luacheck {
             // Existing Luacheck consumers presumably use --formatter plain
@@ -372,20 +413,22 @@ fn start(matches: opts::Options) {
                 eprintln!("`selene generate-roblox-std`.");
 
                 match generate_roblox_std(false) {
-                    Ok(_) => match StandardLibrary::from_config_name(&config.std, Some(&current_dir)) {
-                        Ok(Some(library)) => library,
+                    Ok(_) => {
+                        match StandardLibrary::from_config_name(&config.std, Some(&current_dir)) {
+                            Ok(Some(library)) => library,
 
-                        // This is technically reachable if you edit your config while it is generating.
-                        Ok(None) => {
-                            error!("Standard library was empty after generating roblox standard library, did you edit your config while running selene?");
-                            std::process::exit(1);
-                        }
+                            // This is technically reachable if you edit your config while it is generating.
+                            Ok(None) => {
+                                error!("Standard library was empty after generating roblox standard library, did you edit your config while running selene?");
+                                std::process::exit(1);
+                            }
 
-                        Err(error) => {
-                            error!("Even after generating the `roblox` standard library, we couldn't retrieve the standard library: {}", error);
-                            std::process::exit(1);
+                            Err(error) => {
+                                error!("Even after generating the `roblox` standard library, we couldn't retrieve the standard library: {}", error);
+                                std::process::exit(1);
+                            }
                         }
-                    },
+                    }
 
                     Err(err) => {
                         error!("Could not create roblox standard library: {}", err);
@@ -482,7 +525,7 @@ fn start(matches: opts::Options) {
         log_total(parse_errors, lint_errors, lint_warnings).ok();
     }
 
-    if parse_errors + lint_errors + lint_warnings > 0 {
+    if parse_errors + lint_errors + lint_warnings + pool.panic_count() > 0 {
         std::process::exit(1);
     }
 }
